@@ -1,7 +1,7 @@
 /*---------------------------------------------------------------------------*/
 /* Program to get/set some ACPI settings                                     */
 /* Author : David Leemans   -   http://freeunix.dyndns.org:8000/             */
-/* Last update : 27-11-2005                                                  */
+/* Last update : 31-01-2006                                                  */
 /*---------------------------------------------------------------------------*/
 /*                                                                           */
 /* acpitool.cpp                                                              */
@@ -37,8 +37,8 @@
 
 using namespace std;
 
-#define App_Version  "AcpiTool v0.4.0"
-#define Release_Date "27-Nov-2005"
+#define App_Version  "AcpiTool v0.4.4"
+#define Release_Date "05-Feb-2006"
 
 int Kernel_24 = 0, Kernel_26 = 0;
 
@@ -89,7 +89,7 @@ int Print_ACPI_Info(int show_ac, int show_therm, int show_trip, int show_fan, in
    
 int Do_Battery_Stuff(int show_empty, int info_level, int verbose)
 {
-    Battery_Info *Batt_Info;
+    Battery_Info *Batt_Info[2];
     int Bat_Nr;
     float Remaining_Percentage, Remaining_Time, Battery_Left_Percent;
     int Time_In_Seconds = 0;
@@ -98,40 +98,45 @@ int Do_Battery_Stuff(int show_empty, int info_level, int verbose)
     int Show_Time = 0;
     int Precision = 0;
     int Is_Charging = 0;
+    int Is_Discharging = 0;
+    int AC_Online = 0;
+    int Present_Batteries = 0;
     
-    int Nr_Batteries = Count_Batteries();
+    int Nr_Batteries = Count_Batteries();   // this actually only counts battery entries found in /proc/acpi/battery  //
+    
+    //printf("Number of battery entries found : %d \n", Nr_Batteries);//
     
     if(Nr_Batteries>0)
     {
-        Batt_Info = new Battery_Info;
-		
         for(int i=0; i<Nr_Batteries; i++)
     	{
 	    Bat_Nr = i+1;
+	    Batt_Info[i] = new Battery_Info;
 	    		
-	    Batt_Info->Battery_Present = 0;
-	    bzero(Batt_Info->Remaining_Cap, 10);
-	    bzero(Batt_Info->Design_Cap, 10);
-	    bzero(Batt_Info->LastFull_Cap, 10);
-	    bzero(Batt_Info->Present_Rate, 10);
-	    bzero(Batt_Info->Charging_State, 12);
-	    bzero(Batt_Info->Technology, 13);
-	    bzero(Batt_Info->Model, 13);
-	    bzero(Batt_Info->Serial, 13);
-	    bzero(Batt_Info->Bat_Type, 13); 
+	    Batt_Info[i]->Battery_Present = 0;
+	    bzero(Batt_Info[i]->Remaining_Cap, 10);
+	    bzero(Batt_Info[i]->Design_Cap, 10);
+	    bzero(Batt_Info[i]->LastFull_Cap, 10);
+	    bzero(Batt_Info[i]->Present_Rate, 10);
+	    bzero(Batt_Info[i]->Charging_State, 12);
+	    bzero(Batt_Info[i]->Technology, 13);
+	    bzero(Batt_Info[i]->Model, 13);
+	    bzero(Batt_Info[i]->Serial, 13);
+	    bzero(Batt_Info[i]->Bat_Type, 13); 
 	    // initialize all struct members to blanks --> avoid rubbish in output //
 			
-	    Get_Battery_Info(Bat_Nr, Batt_Info, verbose);
-	    Show_Time = atoi(Batt_Info->Present_Rate);   
+	    Get_Battery_Info(Bat_Nr, Batt_Info[i], verbose);
+	    Show_Time = atoi(Batt_Info[i]->Present_Rate);   
 	    // avoid division by 0 if system is on AC power and battery is full and thus not charging //
 
 	    Is_Charging = 0;  // determine whether battery is charging or not //
 	    
-	    switch(Batt_Info->Battery_Present)
+	    switch(Batt_Info[i]->Battery_Present)
 	    {
 		case 1 : 
 		{	
-	    	    Remaining_Percentage = float(atoi(Batt_Info->Remaining_Cap)) / float(atoi(Batt_Info->LastFull_Cap)) * 100.0;
+	    	    Present_Batteries++;
+		    Remaining_Percentage = float(atoi(Batt_Info[i]->Remaining_Cap)) / float(atoi(Batt_Info[i]->LastFull_Cap)) * 100.0;
 		    
 		    /* from Alan Pope : some broken Dell batteries report a remaining capacity bigger
 		       than their last full capacity or their design capacity. This led acpitool to report
@@ -145,14 +150,21 @@ int Do_Battery_Stuff(int show_empty, int info_level, int verbose)
             	    else
             		Precision = 4;
             	    
-		    if(strncmp(Batt_Info->Charging_State,"char",4)==0) Is_Charging = 1;
+		    if(strncmp(Batt_Info[i]->Charging_State,"char",4)==0) 
+		    {
+			Is_Charging = 1;
+		    }
+		    else
+		    {
+			if(strncmp(Batt_Info[i]->Charging_State,"disch",5)==0) Is_Discharging = 1;
+		    }
 		    		    
 	    	    if(Show_Time)      // calculate remaining or charging time only if present battery rate != 0 //
 	    	    {
 			if(Is_Charging)
-			  Remaining_Time = (float(atoi(Batt_Info->LastFull_Cap)) - float(atoi(Batt_Info->Remaining_Cap))) / float(atoi(Batt_Info->Present_Rate)); 
+			  Remaining_Time = (float(atoi(Batt_Info[i]->LastFull_Cap)) - float(atoi(Batt_Info[i]->Remaining_Cap))) / float(atoi(Batt_Info[i]->Present_Rate)); 
 			else
-			  Remaining_Time = float(atoi(Batt_Info->Remaining_Cap)) / float(atoi(Batt_Info->Present_Rate)); 
+			  Remaining_Time = float(atoi(Batt_Info[i]->Remaining_Cap)) / float(atoi(Batt_Info[i]->Present_Rate)); 
 			// this represents hours //
 		
 			Time_In_Seconds = int(Remaining_Time * 3600.0);
@@ -162,8 +174,8 @@ int Do_Battery_Stuff(int show_empty, int info_level, int verbose)
 			Time_In_Seconds = Time_In_Seconds - (Minutes * 60);
 	    	    }
 		    
-		    if(atoi(Batt_Info->Design_Cap) > 0)
-		      Battery_Left_Percent = float(atoi(Batt_Info->LastFull_Cap)) / float(atoi(Batt_Info->Design_Cap)) * 100.0;
+		    if(atoi(Batt_Info[i]->Design_Cap) > 0)
+		      Battery_Left_Percent = float(atoi(Batt_Info[i]->LastFull_Cap)) / float(atoi(Batt_Info[i]->Design_Cap)) * 100.0;
 		    else
 		      Battery_Left_Percent = -1.0;
 		      			  
@@ -171,7 +183,7 @@ int Do_Battery_Stuff(int show_empty, int info_level, int verbose)
 	    	    switch(info_level)
 	    	    {
 			case 0 : cout.precision(Precision);
-			         cout<<"  Battery #"<<Bat_Nr<<"     : "<<Batt_Info->Charging_State<<", "<<showpoint<<Remaining_Percentage<<"%";
+			         cout<<"  Battery #"<<Bat_Nr<<"     : "<<Batt_Info[i]->Charging_State<<", "<<showpoint<<Remaining_Percentage<<"%";
 			         if(Show_Time)
 			    	    cout<<", "<<setfill('0')<<setw(2)<<Hours<<":"<<setfill('0')<<setw(2)<<Minutes<<":"\
 				    <<setfill('0')<<setw(2)<<Time_In_Seconds;
@@ -179,7 +191,7 @@ int Do_Battery_Stuff(int show_empty, int info_level, int verbose)
 			         break;
 				 
 			case 1 : cout<<"  Battery #"<<Bat_Nr<<"     : present"<<endl;
-			         cout<<"    Remaining capacity : "<<Batt_Info->Remaining_Cap<<", ";
+			         cout<<"    Remaining capacity : "<<Batt_Info[i]->Remaining_Cap<<", ";
 			         cout.precision(Precision);
 			         cout<<showpoint<<Remaining_Percentage<<"%";
 				 
@@ -188,8 +200,8 @@ int Do_Battery_Stuff(int show_empty, int info_level, int verbose)
 				    <<setfill('0')<<setw(2)<<Time_In_Seconds;
 				 cout<<endl;
 				 
-				 cout<<"    Design capacity    : "<<Batt_Info->Design_Cap<<endl;
-				 cout<<"    Last full capacity : "<<Batt_Info->LastFull_Cap;
+				 cout<<"    Design capacity    : "<<Batt_Info[i]->Design_Cap<<endl;
+				 cout<<"    Last full capacity : "<<Batt_Info[i]->LastFull_Cap;
 				 
 				 if(Battery_Left_Percent<100.0)
 				 {
@@ -199,14 +211,14 @@ int Do_Battery_Stuff(int show_empty, int info_level, int verbose)
 				 else
 				   cout<<endl;
 				 
-				 cout<<"    Present rate       : "<<Batt_Info->Present_Rate<<endl;
-			         cout<<"    Charging state     : "<<Batt_Info->Charging_State<<endl;
-			         cout<<"    Battery type       : "<<Batt_Info->Technology<<", "<<Batt_Info->Bat_Type<<endl;
+				 cout<<"    Present rate       : "<<Batt_Info[i]->Present_Rate<<endl;
+			         cout<<"    Charging state     : "<<Batt_Info[i]->Charging_State<<endl;
+			         cout<<"    Battery type       : "<<Batt_Info[i]->Technology<<", "<<Batt_Info[i]->Bat_Type<<endl;
 				 
-				 if (strlen(Batt_Info->Model)!=0)
-			            cout<<"    Model number       : "<<Batt_Info->Model<<endl;
-			         if (strlen(Batt_Info->Serial)!=0)
-				    cout<<"    Serial number      : "<<Batt_Info->Serial<<endl;
+				 if (strlen(Batt_Info[i]->Model)!=0)
+			            cout<<"    Model number       : "<<Batt_Info[i]->Model<<endl;
+			         if (strlen(Batt_Info[i]->Serial)!=0)
+				    cout<<"    Serial number      : "<<Batt_Info[i]->Serial<<endl;
 				 /* on some batteries, these values are empty */     
 				    
 			         break;
@@ -222,12 +234,61 @@ int Do_Battery_Stuff(int show_empty, int info_level, int verbose)
 		     break;
 	    case 3 : cout<<"  Battery         : <not available>"<<endl;
 		     break;
-	    default : cout<<"  Undefined present value, fix this. Value : "<<Batt_Info->Battery_Present<<endl;
+	    default : cout<<"  Undefined present value, fix this. Value : "<<Batt_Info[i]->Battery_Present<<endl;
 	              break;
 	    }    
 	
 	}
-	delete Batt_Info;
+	
+	if(Is_Discharging)
+	    AC_Online = 0;    // assume AC must be  offline if at least 1 battery is discharging //
+	else
+	    AC_Online = 1;    // assume (possibly dangerous) AC is online if all batteries report being charging or charged //
+	
+	/* printf(" Number of batteries present : %d \n", Present_Batteries); 
+	printf(" AC_Online =  %d \n", AC_Online); */
+	
+	// calculate total battery time left if more than 1 battery is present and AC is offline //
+	if(Present_Batteries>1 && AC_Online==0)  
+	{
+	    int Total_Remaining_Cap, Total_LastFull_Cap, Total_Rate;
+	    
+	    Total_LastFull_Cap = 0;
+	    Total_Remaining_Cap = 0;
+	    Total_Rate = 0;
+	    
+	    for(int t=0; t<Present_Batteries; t++)
+	    {
+		Total_Remaining_Cap = Total_Remaining_Cap + (atoi(Batt_Info[t]->Remaining_Cap));
+		Total_LastFull_Cap = Total_LastFull_Cap + (atoi(Batt_Info[t]->LastFull_Cap));
+		Total_Rate = Total_Rate + (atoi(Batt_Info[t]->Present_Rate));
+	    }  
+	    
+	    if(Total_Rate > 0)           // avoid division by 0 if all batteries are charged //
+	    {
+		Remaining_Percentage = float(Total_Remaining_Cap) / float(Total_LastFull_Cap) * 100.0;
+		
+		if( Remaining_Percentage > 100.0) Remaining_Percentage = 100.0;
+                        	    
+		if( int(Remaining_Percentage) < 10)
+            	    Precision = 3;
+		else
+		    Precision = 4;
+		
+		Remaining_Time = float(Total_Remaining_Cap) / float(Total_Rate); 
+		Time_In_Seconds = int(Remaining_Time * 3600.0);
+		Hours = Time_In_Seconds / 3600;
+		Time_In_Seconds = Time_In_Seconds - (Hours * 3600);
+		Minutes = Time_In_Seconds / 60;
+		Time_In_Seconds = Time_In_Seconds - (Minutes * 60);
+		
+		cout.precision(Precision);
+		cout<<"  All batteries  : "<<showpoint<<Remaining_Percentage<<"%";
+		cout<<", "<<setfill('0')<<setw(2)<<Hours<<":"<<setfill('0')<<setw(2)<<Minutes<<":"
+		<<setfill('0')<<setw(2)<<Time_In_Seconds<<endl; 
+	    }
+	}
+	for(int t=0; t<Nr_Batteries; t++) delete Batt_Info[t];
     }
     else
     {
@@ -373,7 +434,7 @@ int Set_Kernel_Version()
 int Do_Thermal_Info(const int show_trip, int verbose)
 {
     ifstream file_in, file_in2;
-    char *dirname, filename[9][65], str[90];
+    char *dirname, filename[15][75], str[120];
     int tz_count = 0;
     DIR *thermal_dir;
     char *name;
@@ -438,7 +499,7 @@ int Do_Thermal_Info(const int show_trip, int verbose)
     	    file_in.open(filename[i-2]);
     	    if(file_in)                   
     	    {
-    		file_in.getline(str, 90);
+    		file_in.getline(str, 120);
     		strncpy(State, str+25, 5);
     		cout<<"  Thermal zone "<<++tz<<" : "<<State<<", ";
     	    }
@@ -446,7 +507,7 @@ int Do_Thermal_Info(const int show_trip, int verbose)
 	    file_in2.open(filename[i-1]);
     	    if(file_in2)                   
     	    {
-		file_in2.getline(str, 90);
+		file_in2.getline(str, 120);
     		strncpy(Temperature, str+25, 5);
     		cout<<Temperature<<endl;
 	    }
@@ -464,8 +525,8 @@ int Do_Thermal_Info(const int show_trip, int verbose)
 		    while(!feof(fp))
 		    {
 			/* fscanf(fp, "%s", str); */
-			bzero(str, 90);
-			fgets(str, 90, fp);
+			bzero(str, 120);
+			fgets(str, 120, fp);
 			if (strlen(str)!=0)  
 			  cout<<"  "<<str;       /* avoid printing empty line */
 		    }
@@ -547,234 +608,286 @@ int Do_AC_Info(int verbose)
 
 int Do_Fan_Info(int verbose)
 {
-    ifstream file_in;
-    char *dirname, filename[4][50], str[40];
-    int fan_count = 0;
-    DIR *fan_dir;
-    struct dirent *fan_dirent;
-    char *name;
-    char FAN_Status[9];
- 
-    if (Has_Thinkpad_ACPI())
-    {
-	Do_Thinkpad_Fan_Info();
-	return 0;
-    }
-    
-    if (Has_Toshiba_ACPI())
-    {
-	Do_Toshiba_Fan_Info();
-	return 0;
-    }
+	ifstream file_in;
+	char *dirname, filename[4][50], str[40];
+	int fan_count = 0;
+	DIR *fan_dir;
+	struct dirent *fan_dirent;
+	char *name;
+	char FAN_Status[9];
 
-    bzero(FAN_Status, 9);         // avoid rubbish in output //
-      
-    dirname = "/proc/acpi/fan/";    
-    fan_dir = opendir(dirname);
-    if(fan_dir)                             
-    {
-    	while((fan_dirent = readdir(fan_dir)))   
-  	{
-	    name = fan_dirent->d_name;
-      	    // skip . and .. // 
-    	    if (!strncmp (".", name, 1) || !strncmp ("..", name, 2)) continue;
-       	    sprintf(filename[fan_count], "/proc/acpi/fan/%s/state", name);
-       	    fan_count++;
-       	}
-    }
-    else        
-    {
-       	if(!verbose)   
-    	{
-    	    cout<<"  Fan            : <not available>"<<endl;
-      	    return 0;
-    	}
-    	else              
-    	{
-    	    cout<<"  Could not open directory : "<<dirname<<endl;
-    	    cout<<"  function Do_Fan_Info() : make sure your kernel has ACPI fan support enabled."<<endl;
-	    return -1;
-    	}	
-    }
-    closedir(fan_dir);  
-    
-    if(fan_count>0)
-    {
-    	for(int i=0; i<fan_count; i++)
-    	{
-    	    file_in.open(filename[i]);
-    	    if(file_in)                   
-    	    {
-		file_in.getline(str, 40);
-    		strncpy(FAN_Status, str+25, 8);
-    		file_in.close();
-    		cout<<"  Fan            : "<<FAN_Status<<endl;
-    	    }
-    	}
-    }
-    else
-    	cout<<"  Fan            : <not available>"<<endl;
-    return 0;
+	if (Has_Thinkpad_ACPI())
+	{
+		Do_Thinkpad_Fan_Info();
+		return 0;
+	}
+
+	if (Has_Toshiba_ACPI())
+	{
+		Do_Toshiba_Fan_Info();
+		return 0;
+	}
+
+	bzero(FAN_Status, 9);         // avoid rubbish in output //
+
+	dirname = "/proc/acpi/fan/";    
+	fan_dir = opendir(dirname);
+	if(fan_dir)                             
+	{
+		while((fan_dirent = readdir(fan_dir)))   
+		{
+			name = fan_dirent->d_name;
+			// skip . and .. // 
+			if (!strncmp (".", name, 1) || !strncmp ("..", name, 2)) continue;
+			sprintf(filename[fan_count], "/proc/acpi/fan/%s/state", name);
+			fan_count++;
+		}
+	}
+	else        
+	{
+		if(!verbose)   
+		{
+			cout<<"  Fan            : <not available>"<<endl;
+			return 0;
+		}
+		else              
+		{
+			cout<<"  Could not open directory : "<<dirname<<endl;
+			cout<<"  function Do_Fan_Info() : make sure your kernel has ACPI fan support enabled."<<endl;
+			return -1;
+		}	
+	}
+	closedir(fan_dir);  
+
+	if(fan_count>0)
+	{
+		for(int i=0; i<fan_count; i++)
+		{
+			file_in.open(filename[i]);
+			if(file_in)                   
+			{
+				file_in.getline(str, 40);
+				strncpy(FAN_Status, str+25, 8);
+				file_in.close();
+				cout<<"  Fan            : "<<FAN_Status<<endl;
+			}
+		}
+	}
+	else
+		cout<<"  Fan            : <not available>"<<endl;
+	return 0;
 }
 
 
+
 int Show_CPU_Info()
 {
-    ifstream file_in, file2_in;
-    char *dirname, filename[12][70], str[200], temp[80];
-    int cpu_count = 0, findex = 0, thrott = 0;
-    DIR *cpu_dir;
-    char *name;
-    
-    file_in.open("/proc/cpuinfo");
-    if(file_in)                   
-    {
-        for(int t=0; t<4; t++) file_in.getline(str, 80);          //skip 4 lines //
-	file_in.getline(str, 100);
-	bzero(temp, 80);
-	strncpy(temp, str+13, 49);
-	cout<<"  CPU type               : "<<temp<<endl;
-	file_in.getline(str, 100);
-	file_in.getline(str, 100);
-	bzero(temp, 80);
-	strncpy(temp, str+11, 10);
-	cout<<"  CPU speed              : "<<temp<<" MHz"<<endl;
-	for(int t=0; t<11; t++) file_in.getline(str,199);        //skip 10 lines //
-	bzero(temp, 80);
-	strncpy(temp, str+11, 10);
-	cout<<"  Bogomips               : "<<temp<<endl;
-	file_in.close();
-    }
-    else
-    {
-	cout<<"  Error : Could not open file /proc/cpuinfo."<<endl;
-	return -1;
-    }	
-       
-    dirname = "/proc/acpi/processor/";    //find all entries in this dir //
-    cpu_dir = opendir(dirname);
-    if(cpu_dir)                     // we can read this dir //
-    {
-	struct dirent **namelist;
-	int n;
+	ifstream file_in, file2_in;
+	char *dirname, filename[12][70], str[200], temp[80];
+	int cpu_count = 0, findex = 0,  thrott = 0;
+	long int max_cstate = 0, cst_cnt = 0, cst_usage_abs[8],
+		 cst_usage_notC0 = 0;
+	long long int bm_activity;
+	float cst_usage_rel[8];
+	DIR *cpu_dir;
+	char *name;
 
-	n = scandir(dirname, &namelist, 0, alphasort);
-	if(n<0)
-	    perror("scandir");
+	file_in.open("/proc/cpuinfo");
+	if(file_in)                   
+	{
+		for(int t=0; t<4; t++) file_in.getline(str, 80);          //skip 4 lines //
+		file_in.getline(str, 100);
+		bzero(temp, 80);
+		strncpy(temp, str+13, 49);
+		cout<<"  CPU type               : "<<temp<<endl;
+		file_in.getline(str, 100);
+		file_in.getline(str, 100);
+		bzero(temp, 80);
+		strncpy(temp, str+11, 10);
+		cout<<"  CPU speed              : "<<temp<<" MHz"<<endl;
+		for(int t=0; t<11; t++) file_in.getline(str,199);        //skip 10 lines //
+		bzero(temp, 80);
+		strncpy(temp, str+11, 10);
+		cout<<"  Bogomips               : "<<temp<<endl;
+		file_in.close();
+	}
 	else
 	{
-	    while(n--)
-	    {
-		name = namelist[n]->d_name;
-		// skip . and .. //
-		if (!strncmp (".", name, 1) || !strncmp ("..", name, 2)) continue;
-		
-		sprintf(filename[findex], "/proc/acpi/processor/%s/info", name);
-		findex++;
-		sprintf(filename[findex], "/proc/acpi/processor/%s/power", name);
-		findex++;
-		sprintf(filename[findex], "/proc/acpi/processor/%s/throttling", name);
-		findex++;
-		cpu_count++;
-				
-		free(namelist[n]);
-	    }
-	    free(namelist);
-	}
-    }      	
-    else
-    {
-   	cout<<"  Function Show_CPU_Info : could not read directory "<<dirname<<endl;
-   	cout<<"  Make sure your kernel has ACPI processor support enabled."<<endl;
-   	return -1;
-    }
-    closedir(cpu_dir);  
+		cout<<"  Error : Could not open file /proc/cpuinfo."<<endl;
+		return -1;
+	}	
 
-    if(cpu_count>0)     
-    {
-    	findex--;
-	
-	if(cpu_count>1) cout<<endl<<"# of CPU's found       : "<<cpu_count<<endl<<endl;
-	
-	for(int i=findex; i>0; i-=3)     //process entries in reverse order //
+
+	dirname = "/proc/acpi/processor/";    //find all entries in this dir //
+	cpu_dir = opendir(dirname);
+	if(cpu_dir)                     // we can read this dir //
 	{
-	    file_in.open(filename[i-2]);
-    	    if(file_in)                   
-    	    {
-	        file_in.getline(str, 90);
-		bzero(temp, 80);
-	        strncpy(temp, str+25, 5);
-		cout<<"  Processor ID           : "<<temp<<endl;
-		
-		file_in.getline(str, 100);
-		file_in.getline(str, 100);
-		bzero(temp, 80);
-	        strncpy(temp, str+25, 5);
-		cout<<"  Bus mastering control  : "<<temp<<endl;
-		
-		file_in.getline(str, 100);
-		bzero(temp, 80);
-	        strncpy(temp, str+25, 5);
-		cout<<"  Power management       : "<<temp<<endl;
-		
-		file_in.getline(str, 100);
-		bzero(temp, 80);
-	        strncpy(temp, str+25, 5);
-		cout<<"  Throttling control     : "<<temp<<endl;
-		if(strncmp(temp,"yes",3)==0)
-    		    thrott = 1;               //yes, this CPU reports it can do power throttling //
-		
-		file_in.getline(str, 100);
-		bzero(temp, 80);
-	        strncpy(temp, str+25, 5);
-		cout<<"  Limit interface        : "<<temp<<endl;
+		struct dirent **namelist;
+		int n;
 
-		file_in.close();
-	    }
-	    else
-	    {
-		cout<<"  could not open file "<<filename[i-2]<<endl;
-	    }	
-	    	    
-	    file2_in.open(filename[i-1]);
-	    if(file2_in)                   
-    	    {
-		file2_in.getline(str, 100);
-		bzero(temp, 80);
-		strncpy(temp, str+25, 5);
-		cout<<"  Active C-state         : "<<temp<<endl;
-		file2_in.close();
-	    }
-	    else
-	    {
-		cout<<"  could not open file "<<filename[i-1]<<endl;
-	    }
-	    
-	    
-	    if(thrott)
-	    {
-		file_in.open(filename[i]);
-		if(file_in)                   
-    		{
-		    file_in.getline(str, 100);
-		    bzero(temp, 80);
-		    strncpy(temp, str+25, 5);
-		    cout<<"  T-state count          : "<<temp<<endl;       // number of throttling states //
-		    file_in.getline(str, 100);
-		    bzero(temp, 80);
-		    strncpy(temp, str+25, 5);
-		    cout<<"  Active T-state         : "<<temp<<endl;
-		    file2_in.close();
+		n = scandir(dirname, &namelist, 0, alphasort);
+		if(n<0)
+			perror("scandir");
+		else
+		{
+			while(n--)
+			{
+				name = namelist[n]->d_name;
+				// skip . and .. //
+				if (!strncmp (".", name, 1) || !strncmp ("..", name, 2)) continue;
+
+				sprintf(filename[findex], "/proc/acpi/processor/%s/info", name);
+				findex++;
+				sprintf(filename[findex], "/proc/acpi/processor/%s/power", name);
+				findex++;
+				sprintf(filename[findex], "/proc/acpi/processor/%s/throttling", name);
+				findex++;
+				cpu_count++;
+
+				free(namelist[n]);
+			}
+			free(namelist);
 		}
-		else cout<<" could not open file "<<filename[i]<<endl;
-	    }
-	    
-	    if(cpu_count>1) cout<<endl;
+	}      	
+	else
+	{
+		cout<<"  Function Show_CPU_Info : could not read directory "<<dirname<<endl;
+		cout<<"  Make sure your kernel has ACPI processor support enabled."<<endl;
+		return -1;
 	}
-    }
-    else   
-	cout<<"  CPU info       : <not available>"<<endl;
-    return 0;
+	closedir(cpu_dir);  
+
+
+	if(cpu_count>0)     
+	{
+		findex--;
+
+		if(cpu_count>1) cout<<endl<<"# of CPU's found       : "<<cpu_count<<endl<<endl;
+
+		for(int i=findex; i>0; i-=3)     //process entries in reverse order //
+		{
+			file_in.open(filename[i-2]); // acpi/processor/%s/info
+			if(file_in)                   
+			{
+				file_in.getline(str, 90); // processor id
+				bzero(temp, 80);
+				strncpy(temp, str+25, 5);
+				cout<<"  Processor ID           : "<<temp<<endl;
+
+				file_in.getline(str, 100); // acpi id
+				file_in.getline(str, 100); // bus mastering control
+				bzero(temp, 80);
+				strncpy(temp, str+25, 5);
+				cout<<"  Bus mastering control  : "<<temp<<endl;
+
+				file_in.getline(str, 100); // power management
+				bzero(temp, 80);
+				strncpy(temp, str+25, 5);
+				cout<<"  Power management       : "<<temp<<endl;
+
+				file_in.getline(str, 100); // throttling control
+				bzero(temp, 80);
+				strncpy(temp, str+25, 5);
+				cout<<"  Throttling control     : "<<temp<<endl;
+				if(strncmp(temp,"yes",3)==0)
+					thrott = 1;		//yes, this CPU reports it can do power throttling //
+
+				file_in.getline(str, 100);	// limit interface (beware: on linux2.4
+											// this line yields "perf mgmt")
+				bzero(temp, 80);
+				strncpy(temp, str+25, 5);
+				cout<<"  Limit interface        : "<<temp<<endl;
+
+				file_in.close();
+			}
+			else
+			{
+				cout<<"  could not open file "<<filename[i-2]<<endl;
+			}	
+
+			file2_in.open(filename[i-1]); // acpi/processor/%s/power
+			if(file2_in)                   
+			{
+				file2_in.getline(str, 100); // active state
+				bzero(temp, 80);
+				strncpy(temp, str+25, 5);
+				cout<<"  Active C-state         : "<<temp<<endl;
+
+			// next code added by Niko Ehrenfeuchter (University of  Freiburg, Germany) //
+			
+				file2_in.getline(str, 100); // max_cstate
+				memset(&temp, 0, sizeof(temp));
+				strncpy(temp, str+26, 2);
+				max_cstate = strtol(temp, NULL, 10);
+				// no output since cstate-count isn't set properly by kernel,
+				// maybe this will be fixed at some later time...
+				// cout << "  Supported C-states     : " << max_cstate << endl;
+
+				file2_in.getline(str, 100); // bus master activity
+				memset(&temp, 0, sizeof(temp));
+				strncpy(temp, str+25, 8);
+				bm_activity = strtoll(temp, NULL, 16);
+				// cout << "  Bus Master Activity hx : 0x" << temp << endl;
+				// cout << "  Bus Master Activity dc : " << bm_activity << endl;
+
+				file2_in.getline(str, 100); // states (line contains no information)
+				while(!file2_in.eof())
+				{
+					file2_in.getline(str, 100);  // cstate Cx
+					if (strlen(str)!=0)
+					{
+						cst_cnt++; // NOTE: we have no usage cnter for C0!!!
+						memset(&temp, 0, sizeof(temp));
+						strncpy(temp, str+80, 8);
+						cst_usage_abs[cst_cnt] = strtol(temp, NULL, 10);
+						cst_usage_notC0 += cst_usage_abs[cst_cnt];
+					}
+				}	
+				// now we can set max_cstate to the correct value
+				max_cstate = cst_cnt;
+				cout << "  C-states (incl. C0)    : " << (max_cstate + 1) << endl;
+
+				// calculate usage etc.
+				for (int cst=1; cst <= max_cstate ; cst++)
+				{
+					cst_usage_rel[cst] = float(cst_usage_abs[cst]) / cst_usage_notC0;
+					fprintf(stdout, "  Usage of state C%d      : %li (%.1f %%)\n", cst,
+							cst_usage_abs[cst], (cst_usage_rel[cst] * 100));
+				}
+
+				file2_in.close();
+			    // end code added by Niko Ehrenfeuchter (University of  Freiburg, Germany) //
+			}
+			else
+			{
+				cout<<"  could not open file "<<filename[i-1]<<endl;
+			}
+
+
+			if(thrott)
+			{
+				file_in.open(filename[i]); // acpi/processor/%s/throttling
+				if(file_in)                   
+				{
+					file_in.getline(str, 100);
+					bzero(temp, 80);
+					strncpy(temp, str+25, 5);
+					cout<<"  T-state count          : "<<temp<<endl;	// number of throttling states //
+					file_in.getline(str, 100);
+					bzero(temp, 80);
+					strncpy(temp, str+25, 5);
+					cout<<"  Active T-state         : "<<temp<<endl;
+					file2_in.close();
+				}
+				else cout<<" could not open file "<<filename[i]<<endl;
+			}
+
+			if(cpu_count>1) cout<<endl;
+		}
+	}
+	else   
+		cout<<"  CPU info       : <not available>"<<endl;
+	return 0;
 }
 
 
@@ -892,12 +1005,14 @@ int Toggle_WakeUp_Device(const int Device, int verbose)
 int Get_Battery_Info(const int bat_nr, Battery_Info *bat_info, int verbose)
 {
     ifstream file_in;
-    char *dirname, filename[4][60], str[100], temp[5];
+    char *dirname, filename[4][65], str[100], temp[5];
     int bat_count = 0, start = 0, findex = 0;
     DIR *battery_dir;
     char *name;
        
-    dirname = "/proc/acpi/battery/";    //find all entries in this dir //
+    dirname = "/proc/acpi/battery/";    //find all entries in this dir 
+    // dirname = "/home/david/dropzone/data/devel/acpi/battery/";    // keep this for testing //    
+
     battery_dir = opendir(dirname);
     if(battery_dir)                     // we can read this dir //
     {
@@ -916,11 +1031,14 @@ int Get_Battery_Info(const int bat_nr, Battery_Info *bat_info, int verbose)
 		if (!strncmp (".", name, 1) || !strncmp ("..", name, 2)) continue;
 		
 		sprintf(filename[findex], "/proc/acpi/battery/%s/info", name);
+		//sprintf(filename[findex], "/home/david/dropzone/data/devel/acpi/battery/%s/info", name);//
 		findex++;
+		
 		sprintf(filename[findex], "/proc/acpi/battery/%s/state", name);
+		//sprintf(filename[findex], "/home/david/dropzone/data/devel/acpi/battery/%s/state", name);//
 		findex++;
+		
 		bat_count++;
-				
 		free(namelist[n]);
 	    }
 	    free(namelist);
@@ -1151,29 +1269,29 @@ int Show_Help()
     " Shows ACPI information from the /proc/acpi filesystem, like battery status,\n"
     " temperature, or ac power. Can also suspend your machine (if supported).\n"
     "\n"
-    "   -a      AC adapter information\n"    //Added by ufoalien:
-    "   -A      show supported Asus ACPI extensions (LCD brightness level, video out routing DSTD/acpi4asus info)\n"
-    "   -b      battery status, available batteries only\n"
-    "   -c      CPU information\n"
-    "   -B      battery status, all info on all battery entries\n"
-    "   -e      show just about everything\n"
-    "   -f      show fan status\n"
-    "   -F x    force fan on (x=1) or switch back to auto mode (x=0). (Toshiba only)\n"
-    "   -h      show this help screen\n"
-    "   -j      eject ultrabay device (Thinkpad only)\n"
-    "   -l x    set LCD brightness level to x, where x is 0..7 (Toshiba and Thinkpad only)\n"
-    "   -m x    switch the mail led on (x=1) or off (x=0) (Asus only)\n"
-    "   -n x    switch the wireless led on (x=1) or off (x=0). (Asus only)\n"
-    "   -o x    set LCD on (x=1) or off (x=0). (Asus only)\n" //added by ufoalien 
-    "   -s      suspend (sleep state S3), if supported\n"
-    "   -S      suspend (sleep state S4), if supported\n"
-    "   -t      thermal information, including trip_points\n"
-    "   -T      show supported Toshiba ACPI extensions (LCD brightness level, video out routing, fan status)\n"
-    "   -v      be more verbose (more detailed error messages, only usefull combined with other options)\n"
-    "   -V      show application version number and release date\n"
-    "   -w      show wakeup capable devices\n"
-    "   -W x    enable/disable wakeup capable device x. The x can be seen when invoking -w first.\n"
-    "   -z x    set Asus LCD brightness level to x, where x is 0..15 (Asus only).\n"       //added by ufoalien 
+    "   -a, --ac_adapter   AC adapter information\n"    //Added by ufoalien:
+    "   -A, --Asus         show supported Asus ACPI extensions (LCD brightness level, video out routing DSTD/acpi4asus info)\n"
+    "   -b                 battery status, available batteries only\n"
+    "   -c, --cpu          CPU information\n"
+    "   -B, --battery      battery status, all info on all battery entries\n"
+    "   -e                 show just about everything\n"
+    "   -f, --fan          show fan status\n"
+    "   -F x               force fan on (x=1) or switch back to auto mode (x=0). (Toshiba only)\n"
+    "   -h, --help         show this help screen\n"
+    "   -j                 eject ultrabay device (Thinkpad only)\n"
+    "   -l x               set LCD brightness level to x, where x is 0..7 (Toshiba and Thinkpad only)\n"
+    "   -m x               switch the mail led on (x=1) or off (x=0) (Asus only)\n"
+    "   -n x               switch the wireless led on (x=1) or off (x=0). (Asus only)\n"
+    "   -o x               set LCD on (x=1) or off (x=0). (Asus only)\n"       //added by ufoalien 
+    "   -s, --suspend      suspend to memory (sleep state S3), if supported\n"
+    "   -S                 suspend to disk (sleep state S4), if supported\n"
+    "   -t, --thermal      thermal information, including trip_points\n"
+    "   -T, --Toshiba      show supported Toshiba ACPI extensions (LCD brightness level, video out routing, fan status)\n"
+    "   -v                 be more verbose (more detailed error messages, only usefull combined with other options)\n"
+    "   -V, --version      show application version number and release date\n"
+    "   -w, --wakeup       show wakeup capable devices\n"
+    "   -W x               enable/disable wakeup capable device x. The x can be seen when invoking -w first.\n"
+    "   -z x               set Asus LCD brightness level to x, where x is 0..15 (Asus only).\n"       //added by ufoalien 
     "\n"
     " If invoked without options, acpitool displays information about available batteries,\n"
     " AC adapter and thermal information.\n\n"
@@ -1190,3 +1308,9 @@ int Show_App_Version()
     cout<<" "<<App_Version<<", "<<"released "<<Release_Date<<endl;
     return 0;
 }
+
+int Usage()
+{
+    cout<<" You invoked acpitool with an unknown option. Type 'acpitool -h' or 'acpitool --help' for info. "<<endl;
+    return 0;
+}
